<?php
/**
 * @package		Donations
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */
 
namespace IPS\donate;

/**
 * Donations Application Class
 */
class _Application extends \IPS\Application
{
	/**
	 * Init
	 *
	 * @return	void
	 */
	public function init()
	{
		/* Can view donations app? */
		if ( !\IPS\Member::loggedIn()->group['g_dt_view'] AND \IPS\Request::i()->do != 'payment' )
		{
			\IPS\Output::i()->error( 'node_error', '', 404, '' );
		}
	}
    
	/**
	 * Install 'other' items.
	 *
	 * @return void
	 */
	public function installOther()
	{
        /* Setup permissions */
        foreach( \IPS\Member\Group::groups( TRUE, TRUE ) as $group )
        {
            /* Admins */
            $adminGroupPermissions = array();
            if( $group->g_access_offline )
            {
                $adminGroupPermissions = array( 'g_dt_moderate_donations' => TRUE );
            }

            /* Now insert group data */
            \IPS\Db::i()->insert( 'donate_groups', array_merge( array( 'group_id' => $group->g_id ), $adminGroupPermissions ), TRUE );
        }

		/* Our custom lang values */
        \IPS\Lang::saveCustom( 'donate', 'donate_goal_1', "Example Goal" );
        \IPS\Lang::saveCustom( 'donate', 'donate_goal_1_desc', "<p>Example goal for new installs.</p><p>Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nullam quis ullamcorper tortor. Nam vel dignissim augue. In sed sollicitudin tellus. Cras vel elit mollis, maximus libero sit amet, ullamcorper justo. Integer congue ut velit a congue. Nullam pulvinar erat sed augue iaculis, vestibulum accumsan nunc venenatis. Aenean non risus eget purus rhoncus vulputate. Quisque mattis, tellus nec mattis mollis, risus dolor egestas nisi, eget commodo tortor sapien ut arcu. </p>" );
        \IPS\Lang::saveCustom( 'donate', 'dt_donate_updates_value', "<p> <span style=\"font-size:16px;\"><strong>Why Donate?</strong></span> </p> <p> <p> Interdum et malesuada fames ac ante ipsum primis in faucibus. Sed dolor velit, aliquam a augue iaculis, malesuada rutrum odio. Donec ac rutrum nisl. Vestibulum rutrum diam a tortor laoreet, sit amet placerat magna viverra. Donec ultricies purus at porttitor dictum. Etiam elit velit, fermentum non ultrices non, mattis non lacus. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos himenaeos. Curabitur pretium, nulla ac aliquet imperdiet, lectus mauris malesuada quam, nec tristique urna arcu at tortor. Pellentesque tortor tortor, dapibus vitae gravida commodo, mollis quis augue. Nam pulvinar scelerisque ornare. Curabitur mattis urna eget accumsan scelerisque. Morbi rhoncus gravida imperdiet. Etiam volutpat, orci a porta consequat, ipsum sem imperdiet enim, at pretium tellus turpis sed turpis. Sed diam turpis, egestas vel erat fermentum, consectetur fringilla turpis. </p> </p> <p> Mauris quis neque a libero imperdiet porta eget nec nunc. In luctus nec neque ut placerat. Etiam blandit justo ante, sed auctor ex faucibus ut. Proin dui libero, lacinia at rhoncus quis, auctor eget urna. Etiam finibus accumsan quam ut mollis. Pellentesque leo justo, rutrum eget efficitur vel, vehicula vitae velit. Pellentesque habitant morbi tristique senectus et netus et malesuada fames ac turpis egestas. Nam dictum diam in bibendum lobortis. </p>" );
        
        /* Add PM custom lang */
		\IPS\Lang::saveCustom( 'donate', 'dt_new_donation_pm_subject_value', "New Donation" );
        \IPS\Lang::saveCustom( 'donate', 'dt_new_donation_pm_message_value', "<p>Hi %member_name%,</p><p>Thank you for your donation of %amount%. We look forward to improving the forums with your donation.</p><p>Thanks %board_name%</p>" );        

        /* Add topic custom lang */
		\IPS\Lang::saveCustom( 'donate', 'dt_topic_title_value', "New Donation %member_name%" );
        \IPS\Lang::saveCustom( 'donate', 'dt_topic_message_value', "<p>Hi %member_name%,</p><p>Thank you for your donation of %amount%. We look forward to improving the forums with your donation.</p><p>Thanks %board_name%</p>" );

        /* Setup topic author */
        \IPS\Settings::i()->changeValues( array( 'dt_topic_author' => \IPS\Member::loggedIn()->member_id ) );

        /* Update settings with first avaliable forum */
        try
        {
            $forumId = \IPS\Db::i()->select( 'id', 'forums_forums', array( 'sub_can_post=?', 1 ) )->first();
            \IPS\Settings::i()->changeValues( array( 'dt_create_topic_forum' => $forumId ) );
        }
        catch( \UnderflowException $e) {}

        /* Update example donation */
        \IPS\Db::i()->update( 'donate_users', array( 'member_id' => \IPS\Member::loggedIn()->member_id, 'member_name' => \IPS\Member::loggedIn()->name ) );
	}      
    
	/**
	 * [Node] Get Icon for tree
	 *
	 * @note	Return the class for the icon (e.g. 'globe')
	 * @return	string|null
	 */
	protected function get__icon()
	{
		return 'dollar';
	}
    
	/**
	 * Default front navigation
	 *
	 * @code
	 	
	 	// Each item...
	 	array(
			'key'		=> 'Example',		// The extension key
			'app'		=> 'core',			// [Optional] The extension application. If ommitted, uses this application	
			'config'	=> array(...),		// [Optional] The configuration for the menu item
			'title'		=> 'SomeLangKey',	// [Optional] If provided, the value of this language key will be copied to menu_item_X
			'children'	=> array(...),		// [Optional] Array of child menu items for this item. Each has the same format.
		)
	 	
	 	return array(
		 	'rootTabs' 		=> array(), // These go in the top row
		 	'browseTabs'	=> array(),	// These go under the Browse tab on a new install or when restoring the default configuraiton; or in the top row if installing the app later (when the Browse tab may not exist)
		 	'browseTabsEnd'	=> array(),	// These go under the Browse tab after all other items on a new install or when restoring the default configuraiton; or in the top row if installing the app later (when the Browse tab may not exist)
		 	'activityTabs'	=> array(),	// These go under the Activity tab on a new install or when restoring the default configuraiton; or in the top row if installing the app later (when the Activity tab may not exist)
		)
	 * @endcode
	 * @return array
	 */
	public function defaultFrontNavigation()
	{
		return array(
			'rootTabs'		=> array( array( 'key' => 'Donate' ) ),
			'browseTabs'	=> array(),
			'browseTabsEnd'	=> array(),
			'activityTabs'	=> array()
		);
	}        
}