//<?php
/**
 * @package		Donations
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	exit;
}

class donate_hook_groupSource extends _HOOK_CLASS_
{
    /**
     * Construct ActiveRecord from database row
     *
     * @param	array	$data							Row from database table
     * @param	bool	$updateMultitonStoreIfExists	Replace current object in multiton store if it already exists there?
     * @return	static
     */
    public static function constructFromData( $data, $updateMultitonStoreIfExists = TRUE )
    {
	try
	{
	        /* Remove our permissions when copying */
	        if( \IPS\Dispatcher::hasInstance() and \IPS\Dispatcher::i()->controllerLocation == 'admin' and isset( \IPS\Request::i()->do ) and \IPS\Request::i()->do == 'copy' )
	        {
	            $ourPermissions = array( 'g_dt_view', 'g_dt_donate', 'g_dt_view_goals', 'g_dt_view_donations', 'g_dt_view_topdonors', 'g_dt_send_donations', 'g_dt_moderate_donations' );
	
	            foreach( $ourPermissions as $perm )
	            {
	                if( isset( $data[ $perm ] ) )
	                {
	                    unset( $data[ $perm ] );
	                }
	            }
	        }
	
	        return parent::constructFromData( $data, $updateMultitonStoreIfExists );
	}
	catch ( \RuntimeException $e )
	{
		if ( method_exists( get_parent_class(), __FUNCTION__ ) )
		{
			return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
		}
		else
		{
			throw $e;
		}
	}
    }
    /**
     * Group datastore
     *
     * @return	array
     */
    static public function getStore()
    {
	try
	{
	        /* Get data store */
	        $store = parent::getStore();
	
	        /* Setup */
	        $newStore = array();
	        $slice = \array_slice( $store, 0, 1 );
	
	        /* Extra data missing? */
	        if( !isset( $slice[ 0 ]['g_dt_view'] ) )
	        {
	            /* Get donate group permissions */
	            $extraData = iterator_to_array( \IPS\Db::i()->select( 'donate_groups.*', 'donate_groups' )->setKeyField( 'group_id' ) );
	
	            foreach( $store as $id => $data )
	            {
	                /* Check exists */
	                if( isset( $extraData[ $id ] ) )
	                {
	                    unset( $extraData[ $id ]['group_id'] );
	                    $newStore[ $id ] = array_merge( $data, $extraData[ $id ] );
	                }
	                else
	                {
	                    $newStore[ $id ] = $data;
	                }
	            }
	        }
	
	        /* Update data store if extra data added */
	        if( \count( $newStore ) )
	        {
	            \IPS\Data\Store::i()->groups = $newStore;
	        }
	
	        return $store;
	}
	catch ( \RuntimeException $e )
	{
		if ( method_exists( get_parent_class(), __FUNCTION__ ) )
		{
			return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
		}
		else
		{
			throw $e;
		}
	}
    }

    /**
     * [ActiveRecord] Delete Record
     *
     * @return	void
     */
    public function delete()
    {
	try
	{
	        \IPS\Db::i()->delete( 'donate_groups', array( 'group_id = ?', $this->g_id ) );
	
	        return parent::delete();
	}
	catch ( \RuntimeException $e )
	{
		if ( method_exists( get_parent_class(), __FUNCTION__ ) )
		{
			return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
		}
		else
		{
			throw $e;
		}
	}
    }
}
