//<?php
/**
 * @package		Donations
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	exit;
}

class donate_hook_memberSource extends _HOOK_CLASS_
{
    /**
     * Donation Totals Cache
     */
    protected $_donateDonations = NULL;

    /**
     * Donation Amount Cache
     */
    protected $_donateAmount = NULL;

    /**
     * Get donation totals
     *
     * @return	string
     */
    public function get_donateDonations()
    {
	try
	{
	        /* Cache donate donations value */
	        if ( $this->_donateDonations === NULL )
	        {
	            try
	            {
	                $this->_donateDonations = \IPS\Db::i()->select( 'donate_donations', 'donate_members', array( 'donate_member_id=?', $this->member_id ) )->first();
	            }
	            catch ( \UnderflowException $e )
	            {
	                $this->_donateDonations = 0;
	            }
	        }
	
	        return $this->_donateDonations;
	}
	catch ( \RuntimeException $e )
	{
		if ( method_exists( get_parent_class(), __FUNCTION__ ) )
		{
			return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
		}
		else
		{
			throw $e;
		}
	}
    }

    /**
     * Set donate totals
     *
     * @param	string	$value	Value
     * @return	void
     */
    public function set_donateDonations( $value )
    {
	try
	{
	        \IPS\Db::i()->insert( 'donate_members', array( 'donate_member_id' => $this->member_id, 'donate_donations' => $value ), TRUE );
	}
	catch ( \RuntimeException $e )
	{
		if ( method_exists( get_parent_class(), __FUNCTION__ ) )
		{
			return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
		}
		else
		{
			throw $e;
		}
	}
    }

    /**
     * Get donation amount
     *
     * @return	string
     */
    public function get_donateAmount()
    {
	try
	{
	        /* Cache donate amount value */
	        if ( $this->_donateAmount === NULL )
	        {
	            try
	            {
	                $this->_donateAmount = \IPS\Db::i()->select( 'donate_amount', 'donate_members', array( 'donate_member_id=?', $this->member_id ) )->first();
	            }
	            catch ( \UnderflowException $e )
	            {
	                $this->_donateAmount = '0.00';
	            }
	        }
	
	        return $this->_donateAmount;
	}
	catch ( \RuntimeException $e )
	{
		if ( method_exists( get_parent_class(), __FUNCTION__ ) )
		{
			return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
		}
		else
		{
			throw $e;
		}
	}
    }

    /**
     * Set donate amount
     *
     * @param	string	$value	Value
     * @return	void
     */
    public function set_donateAmount( $value )
    {
	try
	{
	        \IPS\Db::i()->insert( 'donate_members', array( 'donate_member_id' => $this->member_id, 'donate_amount' => $value ), TRUE );
	}
	catch ( \RuntimeException $e )
	{
		if ( method_exists( get_parent_class(), __FUNCTION__ ) )
		{
			return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
		}
		else
		{
			throw $e;
		}
	}
    }
}
