<?php


namespace IPS\donate\modules\front\donate;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * ajax
 */
class _ajax extends \IPS\Dispatcher\Controller
{
    /**
     * Top Donors
     *
     * @retun	void
     */
    public function topDonors()
    {
        /* Can view? */
        if( !\IPS\Member::loggedIn()->group['g_dt_view_topdonors'] )
        {
            return '';
        }

        /* How many? */
        $limit = \intval( ( isset( \IPS\Request::i()->limit ) and \IPS\Request::i()->limit <= 30 ) ? \IPS\Request::i()->limit : 5 );

        /* What timeframe? */
        $where = array( arraY( 'member_id>0 AND anon=0 AND anon_amount=0' ) );
        $timeframe = 'all';
        if ( isset( \IPS\Request::i()->time ) and \IPS\Request::i()->time != 'all' )
        {
            switch ( \IPS\Request::i()->time )
            {
                case 'week':
                    $where[] = array( 'date>?', \IPS\DateTime::create()->sub( new \DateInterval( 'P1W' ) )->getTimestamp() );
                    $timeframe = 'week';
                    break;
                case 'month':
                    $where[] = array( 'date>?', \IPS\DateTime::create()->sub( new \DateInterval( 'P1M' ) )->getTimestamp() );
                    $timeframe = 'month';
                    break;
                case 'year':
                    $where[] = array( 'date>?', \IPS\DateTime::create()->sub( new \DateInterval( 'P1Y' ) )->getTimestamp() );
                    $timeframe = 'year';
                    break;
            }

            $topDonors = iterator_to_array( \IPS\Db::i()->select( 'member_id as member, SUM(amount*rate) as amount', 'donate_users', $where, 'amount DESC', $limit, 'member' )->setKeyField('member')->setValueField('amount') );
        }
        else
        {
            $topDonors = iterator_to_array( \IPS\Db::i()->select( 'member_id as member, SUM(amount*rate) as amount', 'donate_users', array( array( 'member_id>0 AND anon=0 AND anon_amount=0' ) ), 'amount DESC', $limit, 'member' )->setKeyField('member')->setValueField('amount') );
        }

        /* Load their data */
        foreach ( \IPS\Db::i()->select( '*', 'core_members', \IPS\Db::i()->in( 'member_id', array_keys( $topDonors ) ) ) as $member )
        {
            \IPS\Member::constructFromData( $member );
        }

        /* Render */
        $output = \IPS\Theme::i()->getTemplate( 'widgets' )->donateTopDonorsRows( $topDonors, $timeframe, \IPS\Request::i()->orientation );
        if ( \IPS\Request::i()->isAjax() )
        {
            \IPS\Output::i()->sendOutput( $output );
        }
        else
        {
            \IPS\Output::i()->output = $output;
        }
    }
}