<?php
/**
 * @package		Donations
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */

namespace IPS\donate\modules\front\donate;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * index
 */
class _index extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 */
	public function execute()
	{
		parent::execute();

		/* Latest goals RSS feed link */
		if( \IPS\Settings::i()->dt_enable_goal_rss AND \IPS\Member::loggedIn()->group['g_dt_view_goals'] )
		{
			\IPS\Output::i()->rssFeeds[ 'latest_goals_rss' ] = \IPS\Http\Url::internal( 'app=donate&module=donate&controller=browse&do=goalsRSS', 'front', 'donate_goalsrss' );
		} 
        
		/* Latest donations RSS feed link */
		if( \IPS\Settings::i()->dt_enable_donation_rss AND \IPS\Member::loggedIn()->group['g_dt_view_donations'] )
		{
			\IPS\Output::i()->rssFeeds[ 'latest_donations_rss' ] = \IPS\Http\Url::internal( 'app=donate&module=donate&controller=browse&do=donationsRSS', 'front', 'donate_donationsrss' );
		}              
        
		/* You must purchase copyright removal before removing */
        if( !\IPS\Settings::i()->devfuse_copy_num && !\IPS\Request::i()->isAjax() )
        {
            \IPS\Output::i()->output .= "<div style='clear:both;text-align:center;position:absolute;bottom:15px;width:95%;'><a href='https://www.devfuse.com/' class='ipsType_light ipsType_smaller'>IP.Board Donations by DevFuse</a></div>";
        }
	}

	/**
	 * Overview page
	 */
	protected function manage()
	{
	    $tabs = array();

        /* Setup index tabs */
        if( \IPS\Settings::i()->dt_enable_updates )
        {
            $tabs['updates'] = array( 'title' => \IPS\Member::loggedIn()->language()->addToStack( 'donate_tab_updates' ),
                'url'   => \IPS\Http\Url::internal( 'app=donate&module=donate&controller=index&tab=updates', 'front', 'donate_tab_updates' )
            );
        }

        if( \IPS\Member::loggedIn()->group['g_dt_view_goals'] )
        {
            $tabs['goals'] = array( 'title' => \IPS\Member::loggedIn()->language()->addToStack( 'donate_tab_goals' ),
                'url'   => \IPS\Http\Url::internal( 'app=donate&module=donate&controller=index&tab=goals', 'front', 'donate_tab_goals' )
            );
        }

        if( \IPS\Member::loggedIn()->group['g_dt_view_donations'] )
        {
            $tabs['donations'] = array( 'title' => \IPS\Member::loggedIn()->language()->addToStack( 'donate_tab_donations' ),
                'url'   => \IPS\Http\Url::internal( 'app=donate&module=donate&controller=index&tab=donations', 'front', 'donate_tab_donations' )
            );
        }

        if( \IPS\Member::loggedIn()->group['g_dt_view_topdonors'] )
        {
            $tabs['topdonors'] = array( 'title' => \IPS\Member::loggedIn()->language()->addToStack( 'donate_tab_topdonors' ),
                'url'   => \IPS\Http\Url::internal( 'app=donate&module=donate&controller=index&tab=topdonors', 'front', 'donate_tab_topdonors' )
            );
        }

        /* What tab are we viewing? */
        if( !isset( \IPS\Request::i()->tab ) or !array_key_exists( \IPS\Request::i()->tab, $tabs ) )
        {
            /* Get default tab and check selected tabs can be accessed */
            $defaultTab = \IPS\Settings::i()->dt_default_tab;
            $defaultTab = ( ( $defaultTab == 'updates' AND \IPS\Settings::i()->dt_enable_updates ) OR ( $defaultTab == 'goals' AND \IPS\Member::loggedIn()->group['g_dt_view_goals'] ) OR ( $defaultTab == 'donations' AND \IPS\Member::loggedIn()->group['g_dt_view_donations'] ) ) ? $defaultTab : 'new';

            $tab = $defaultTab;
        }
        else
        {
            $tab = \IPS\Request::i()->tab;
        }

        /* Work out the tab content */
        $tabContent = '';
        if( $tab == 'updates' AND \IPS\Settings::i()->dt_enable_updates )
        {
            /* Output to tab */
            $tabContent = \IPS\Theme::i()->getTemplate( 'browse' )->donateTab( $tab, \IPS\Theme::i()->getTemplate( 'global', 'core' )->box( \IPS\Member::loggedIn()->language()->addToStack('dt_donate_updates_value'), array( 'ipsPad ipsType_normal' ) ) );
        }

        if( $tab == 'goals' AND \IPS\Member::loggedIn()->group['g_dt_view_goals'] )
        {
            /* Get goals */
            $table = new \IPS\Helpers\Table\Db( 'donate_goals', \IPS\Http\Url::internal( 'app=donate&module=donate&controller=index&tab=goals', 'front', 'donate_tab_goals' ) );
            $table->rowsTemplate  = array( \IPS\Theme::i()->getTemplate( 'browse' ), 'goalRow' );
            $table->langPrefix    = "donate_";
            $table->limit         = 5;
            $table->noModerate    = TRUE;

            /* Column stuff */
            $table->include = array( 'g_id', 'g_amount', 'g_show' );
            $table->mainColumn = 'g_id';

            $table->sortOptions['g_views'] = 'g_views';
            $table->sortOptions['g_amount'] = 'g_amount';
            $table->sortOptions['g_donations'] = 'g_donations';

            /* Set default filter if none provided */
            if( !isset( \IPS\Request::i()->filter ) AND !\IPS\Request::i()->isAjax() )
            {
                $table->filter = "filter_active_goals";
            }

            /* Filters */
            $table->filters = array(
                'filter_active_goals' => 'g_show=1',
                'filter_closed_goals' => 'g_show=0',
            );

            $table->parsers = array(
                'g_id' => function( $val, $row )
                {
                    /* Yes it's possible */
                    if( !$val )
                    {
                        return NULL;
                    }

                    try
                    {
                        return \IPS\donate\Goal::load( $val );
                    }
                    catch( \OutOfRangeException $ex )
                    {
                        return NULL;
                    }
                }
            );

            /* Output to tab */
            $tabContent = \IPS\Theme::i()->getTemplate( 'browse' )->donateTab( $tab, (string) $table );
        }
        elseif( $tab == 'donations' AND \IPS\Member::loggedIn()->group['g_dt_view_donations'] )
        {
            /* Get donations */
            $table = new \IPS\Helpers\Table\Content( 'IPS\donate\Donation', \IPS\Http\Url::internal( 'app=donate&module=donate&controller=index&tab=donations', 'front', 'donate_tab_donations' ), NULL, NULL, FALSE, 'read', FALSE );
            $table->rowsTemplate  = array( \IPS\Theme::i()->getTemplate( 'browse' ), 'donationRow' );
            $table->langPrefix    = "donate_";
            $table->limit         = 10;
            $table->noModerate    = TRUE;
            $table->sortOptions   = array();
            $table->sortBy        = 'date';
            $table->sortDirection = 'desc';
            $table->where[]       = array( 'status=1' );

            /* Set default filter if none provided */
            if( !isset( \IPS\Request::i()->filter )  )
            {
                $table->filter = "filter_active_donations";
            }

            /* Filters */
            $table->filters = array(
                'filter_active_donations' => array( 'date>?', (int) \IPS\Settings::i()->dt_donation_reset ),
                'filter_note_left' => 'note>0',
            );

            /* Output to tab */
            $tabContent = \IPS\Theme::i()->getTemplate( 'browse' )->donateTab( $tab, (string) $table );
        }
        elseif( $tab == 'topdonors' AND \IPS\Member::loggedIn()->group['g_dt_view_topdonors'] )
        {
            $donations = array();

            /* Setup where array */
            $where[] = array( 'member_id>?', 0 );
            $where[] = array( 'status=?', 1 );
            $where[] = array( 'anon=?', 0 );
            $where[] = array( 'anon_amount=?', 0 );

            /* Only show 'active' donations? */
            if( \IPS\Request::i()->filter )
            {
                $where[] = array( 'date>?', (int) \IPS\Settings::i()->dt_donation_reset );
            }

            /* Get all top donors */
            $donations = \IPS\Db::i()->select( 'member_id as member, SUM(amount*rate) AS total_amount, count(amount) AS donations', 'donate_users', $where, 'total_amount DESC', 100, array( 'member_id' ) )->setKeyField('member');

            /* Setup table */
            $table = new \IPS\Helpers\Table\Custom( $donations, \IPS\Http\Url::internal( 'app=donate&module=donate&controller=index&tab=topdonors', 'front', 'donate_tab_topdonors' ) );
            $table->rowsTemplate = array( \IPS\Theme::i()->getTemplate( 'browse' ), 'topDonorRow' );
            $table->classes[] = 'ipsGrid ipsPad_half';
            $table->langPrefix = "donate_";
            $table->limit = 16;

            /* Filters */
            $table->filters = array(
                'filter_active_donations' => array( 'date>?', (int) \IPS\Settings::i()->dt_donation_reset ),
            );

            $tabContent = \IPS\Theme::i()->getTemplate( 'browse' )->donateTab( $tab, $table );
        }

        /* Display tab request on ajax */
        if( \IPS\Request::i()->isAjax() and isset( \IPS\Request::i()->tab ) )
        {
            \IPS\Output::i()->sendOutput( $tabContent );
        }
        
		/* Set Online Location */
		$permissions = \IPS\Dispatcher::i()->module->permissions();
		\IPS\Session::i()->setLocation( \IPS\Http\Url::internal( 'app=donate&module=donate&controller=index', 'front', 'donate' ), explode( ',', $permissions['perm_view'] ), 'loc_donate_index' );
       
		/* Display */
		\IPS\Output::i()->title	= \IPS\Member::loggedIn()->language()->addToStack('__app_donate');
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'browse' )->index( $tabs, $tab, $tabContent );
	}
    
	/**
	 * Incoming Payment
	 *
	 * @return	void
	 */
	protected function payment()
	{    
	  	# Need data to process
		if( empty( $_POST ) && empty( $_GET ) )
		{
     		$this->_logError( 'donationlog__emptypost', 'donationlog__emptypost_desc' );		  
		}    

        /* Get gateway details */
		try
		{
			$gateway = \IPS\donate\Gateway::load( \IPS\Request::i()->gateway ); 
		}
		catch( \Exception $e )
        {
            $this->_logError( 'donationlog__gateway', $e );
		}       
        
        /* Process donation */
		try
		{
			$donation = $gateway->auth( $_POST );
		}
		catch( \Exception $e )
        {
			$this->_logError( 'donationlog__auth', $e );
		}        
		try
		{
            $donation = $gateway->process( $donation );
		}
		catch( \Exception $e )
        {
			$this->_logError( 'donationlog__process', $e );
		}

		/* Check input is a donation from this app */
		if( ( isset( $donation['name'] ) AND \IPS\Member::loggedIn()->language()->checkKeyExists( 'forum_donation' ) ) && $donation['name'] != \IPS\Member::loggedIn()->language()->get( 'forum_donation' ) )
        {
            $this->_logError( 'donationlog__donation_mismatch', 'donationlog__donation_mismatch_desc', NULL );
        }

        /* No invoice, no processing */
        if( !isset( $donation['invoice'] ) OR !$donation['invoice'] )
        {
            $this->_logError( 'donationlog__invoice', 'donationlog__no_invoice' );
        }

        /* Get invoice details */
        try
        {
            $invoice = \IPS\donate\Invoice::load( $donation['invoice'] );
        }
        catch ( \OutOfRangeException $e )
        {
            $this->_logError( 'donationlog__invoice', $e );
        }

        if( $invoice->status == 1 )
        {
            $this->_logError( 'donationlog__already_paid', 'donationlog__already_paid_desc' );
        }

        /* Get member details if member */
        $member = NULL;
        
        if( isset( $donation['member_id'] ) AND $donation['member_id'] )
        {        
    		try
    		{
    			$member = \IPS\Member::load( $donation['member_id'] );
    		}
			catch ( \OutOfRangeException $e ) 
            {
                $this->_logError( 'donationlog__member', $e );
			}                        
        }         
        
        /* Get goal details */
        if( isset( $invoice->goal ) AND $invoice->goal )
        {        
    		try
    		{
    			$goal = \IPS\donate\Goal::load( $invoice->goal );
    		}
			catch ( \OutOfRangeException $e ) 
            {
                $this->_logError( 'donationlog__goal', $e, $member );
			} 
        }        
        
        /* Check againgst goal email */
        if( $goal->emails )
        {
            if( $gateway->file != 'offline' )
            {
                if( mb_strtolower( $donation['gateway_email'] ) != mb_strtolower( trim( $goal->emails ) ) && mb_strtolower( $donation['gateway_receiver'] ) != mb_strtolower( trim( $goal->emails ) ) )
                {
                    $this->_logError( 'donationlog__goalemail_mismatch', 'donationlog__goalemail_mismatch_desc', $member );
                }
            }
        }
        else
        {
            if( $gateway->emails ) 
            {
                $gatewayEmails = json_decode( $gateway->emails, TRUE );    
                
                foreach( $gatewayEmails as $address )
                {
                    $emailMatches[ $address['value'] ] = mb_strtolower( $address['value'] );         
                } 
    
                /* Check extra emails are valid */
                if( !\in_array( mb_strtolower( $donation['gateway_email'] ), $emailMatches ) && !\in_array( mb_strtolower( $donation['gateway_receiver'] ), $emailMatches ) )
                {
        			$this->_logError( 'donationlog__gateway_mismatch', 'donationlog__gateway_mismatch_desc', $member );            
                }                
            }
            else
            {
                /* Check default gateway address or id matches */
                if( mb_strtolower( $donation['gateway_email'] ) != mb_strtolower( trim( $gateway->email ) ) && mb_strtolower( $donation['gateway_receiver'] ) != mb_strtolower( trim( $gateway->email ) ) )
                {
        			$this->_logError( 'donationlog__gateway_mismatch', 'donationlog__gateway_mismatch_desc', $member );            
                }            
            }            
        }

        /* Check for duplicate txn id */
        $existingDonation = \IPS\Db::i()->select( 'txn_id', 'donate_users', array( array( 'txn_id=?', $donation['txn_id'] ) ) )->count();
        
        if( $existingDonation )
        {
			$this->_logError( 'donationlog__txn', 'donationlog__txn_desc', $member );     
        }

        /* Get curreny details */
        if( isset( $donation['currency'] ) AND $donation['currency'] )
        {
			try
			{
			    $currency = \IPS\donate\Currency::load( $donation['currency'], 'c_tag' );
			}
			catch ( \OutOfRangeException $e ) 
            {
                $this->_logError( 'donationlog__currency', $e, $member );
			}             
        }
        
        /* Save donation to database */ 
		try
		{
    		$save = new \IPS\donate\Donation;
    		$save->status	   = (int) $donation['status'];
    		$save->member_id   = (int) $member->member_id;
            $save->member_name = $member->name ? $member->name : 'Guest';
            $save->txn_id	   = $donation['txn_id'];
    		$save->date	       = time();
    		$save->amount	   = $donation['amount'];
    		$save->fees	       = $donation['fees'];
    		$save->currency	   = isset( $currency ) ? $currency->_id : 0;
            $save->rate	       = isset( $currency ) ? $currency->rate : 1;
    		$save->goal	       = isset( $goal ) ? $goal->_id : 0;
    		$save->note	       = ( isset( $donation['note'] ) AND $donation['note'] ) ? $donation['note'] : '';  
    		$save->anon	       = (int) $donation['anonymous'];  
    		$save->anon_amount = (int) $donation['anonymous_amount'];
            $save->approved_notified = ( $donation['status'] ) ? time() : 0;
            $save->save();
            
            /* Log non error */
     		$log = new \IPS\donate\Donation\Log;
    		$log->problem = \IPS\Member::loggedIn()->language()->get( 'donationlog__success' );

            /* Mark invoice as paid */
            $invoice->status  = 1;
            $invoice->donation = $save->id;
            $invoice->gateway  = $gateway->id;
            $invoice->save();

            /* Assign custom values to donation now */
            \IPS\Db::i()->update( 'donate_custom_values', array( 'donation_id' => $save->id ), array( 'invoice_id=?', $invoice->id ) );

            /* Make donor log author */
            if( isset( $member ) AND $member )
            {
        	    $log->author_id   = (int) $member->member_id;
                $log->author_name = $member->name;  
            }      
                                                                     
    		$log->save();            
            
            /* Build goal stats */
            if( isset( $goal ) )
            {
                $goal->rebuildGoalStats();   
            }            
		}
		catch ( \Exception $e ) 
        {
            $this->_logError( 'donationlog__donation', $e, $member );
		}        

        /* Run a few more member tasks */
        if( isset( $member ) AND $member )
        {
            /* Promote group if donation approved */
            if( $donation['status'] )
            {
        		try
        		{
                    \IPS\donate\Reward::issue( $donation['amount'], $donation['currency'], $member );
        		}
        		catch ( \Exception $e ) 
                {
                    $this->_logError( 'donationlog__success', $e, $member );
        		}                
            }
            
            /* Update donor count */
    		try
    		{
	            /* Increase if approved donation */
    		    if( $donation['status'] )
                {
                    $member->donateDonations = $member->donateDonations + 1;
                    $member->donateAmount    = $member->donateAmount + $donation['amount'];
                }
    		}
    		catch( \Exception $e )
            {
                $this->_logError( 'donationlog__member', $e, $member ); 
    		}                         
        }
        
        /* Show offline donors message */
        if( $gateway->file == 'offline' )
        {
            return \IPS\Output::i()->output = \IPS\Theme::i()->getTemplate('payment')->paymentMessage();          
        }        

        exit();
    }
    
	/**
	 * Log Payment Error
	 */
	protected function _logError( $problem=NULL, $error=NULL, $member=NULL )
	{ 
	    /* Get lang strings for errors */
		if( \IPS\Member::loggedIn()->language()->checkKeyExists( $problem ) )
		{
			$problem = \IPS\Member::loggedIn()->language()->get( $problem );
		}	   

		if( !preg_match( '/[^A-Za-z0-9_]/', $error ) && \IPS\Member::loggedIn()->language()->checkKeyExists( $error ) )
		{
			$error = \IPS\Member::loggedIn()->language()->get( $error );
		}
        
        /* Log error */
 		$log = new \IPS\donate\Donation\Log;
		$log->problem   = $problem;         
		$log->post_data = $error;  
        
        /* Use donor member details if present */
        if( $member )
        {
            $log->author_id   = (int) $member->member_id;
            $log->author_name = $member->name;
            $log->ip_address  = $member->ip_address;
        }
                                                        
		$log->save();
        
        /* Display error */
        print $problem;
        exit();
    }           	   
}