<?php
/**
 * @package		Donations
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */

namespace IPS\donate\modules\front\donate;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * payment
 */
class _payment extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{	
		parent::execute();

	    /* Can donate? */
        if( !\IPS\Member::loggedIn()->group['g_dt_donate'] )	
        {
            \IPS\Output::i()->error( 'donate_error', '', 403, '' );    
        }        

		/* You must purchase copyright removal before removing */
        if( !\IPS\Settings::i()->devfuse_copy_num && !\IPS\Request::i()->isAjax() )
        {
            \IPS\Output::i()->output .= "<div style='clear:both;text-align:center;position:absolute;bottom:15px;width:95%;'><a href='https://www.devfuse.com/' class='ipsType_light ipsType_smaller'>IP.Board Donations by DevFuse</a></div>";
        }        
	}

	/**
	 * Payment Screen
	 *
	 * @return	void
	 */
	protected function manage()
	{
	    /* Temp cache the goal id so can use in wizard form */
        if( isset( \IPS\Request::i()->goal ) AND \IPS\Request::i()->goal )
        {
            \IPS\Data\Cache::i()->storeWithExpire( 'donationGoalSelect_'.\IPS\Member::loggedIn()->member_id, \IPS\Request::i()->goal, \IPS\DateTime::create()->add( new \DateInterval('P1D') ), TRUE );
        }

        /* Invoice already generated? */
        try
        {
            $invoice = \IPS\donate\Invoice::constructFromData( \IPS\Db::i()->select( '*', 'donate_invoices', ( \IPS\Member::loggedIn()->member_id ) ? array( "invoice_status=0 AND invoice_member=?", \IPS\Member::loggedIn()->member_id ) : array( "invoice_status=0 AND invoice_ip=?", \IPS\Request::i()->ipAddress() ) )->first() );
        }
        /* Generate new invoice */
        catch( \UnderflowException $e )
        {
            $invoice = new \IPS\donate\Invoice;
            $invoice->save();
        }

	    /* Set up the step array */
		$steps = array();
        		
        /* Setup first basic step */		
		$steps['setup'] = function( $data ) use ( $invoice )
		{
		    /* Get cached goal selection */
            $donateGoal = NULL;
            try
            {
                $donateGoal = (int) \IPS\Data\Cache::i()->getWithExpire( 'donationGoalSelect_'. \IPS\Member::loggedIn()->member_id, TRUE );
            }
            catch( \OutOfRangeException $e ){}

      		/* Basic form */
    		$form = new \IPS\Helpers\Form( 'basic', 'continue' );
    		$form->class = ' ipsForm_fullWidth ipsPad';

            /* Build currency list */
            foreach( \IPS\donate\Currency::roots() as $currency )
            {
                /* Save first currency if only one */
                $firstCurrency = $currency;

                /* Add to list */
                $currencyOptions[ $currency->tag ] = $currency->_title;
            }

            /* Amount select or enter */
            if( \IPS\Settings::i()->dt_amounts )
            {
                /* Setup amount dropdown */
                $amountDropdown = array();
                foreach( explode( ',', \IPS\Settings::i()->dt_amounts ) as $amount )
                {
                    $amountDropdown[ $amount ] = ( \count( $currencyOptions ) == 1 ) ? $firstCurrency->symbol . $amount : $amount;
                }

                $form->add( new \IPS\Helpers\Form\Select( 'amount', NULL, TRUE, array( 'options' => $amountDropdown ) ) );
            }
            else
            {
                $amountField = new \IPS\Helpers\Form\Text( 'amount', NULL, TRUE, array( 'placeholder' => \IPS\Member::loggedIn()->language()->addToStack('enter_amount') ) );

                if( \count( $currencyOptions ) == 1 )
                {
                    $amountField->description = \IPS\Member::loggedIn()->language()->addToStack('donate_amount_prefix', TRUE, array( 'returnBlank' => TRUE, 'htmlsprintf' => array( $firstCurrency->symbol ) ) );
                }

                $form->add( $amountField );
            }


            /* No point giving list if only 1 currency shown */
            if( \count( $currencyOptions ) > 1 )
            {
                $form->add( new \IPS\Helpers\Form\Select( 'currency', TRUE, TRUE, array( 'options' => $currencyOptions ) ) );            
            }  
            else
            {
                $form->hiddenValues['currency'] = $firstCurrency->tag;
            }

            /* Build goal list */
            $goalOptions = array();
            
    		foreach( \IPS\donate\Goal::roots( NULL, NULL, array( array( 'g_show=1' ) ) ) as $goal )
    		{
    			$goalOptions[ $goal->id ] = $goal->_title;
    		}             
            
            /* Add goal select */
            if( \count( $goalOptions ) )
            {
                $form->add( new \IPS\Helpers\Form\Select( 'goal', $donateGoal, FALSE, array( 'options' => $goalOptions ) ) );
            }  
            else
            {
                $form->hiddenValues['goal'] = 0;
            }            
            
            /* Build gateway list */
            $gatewayOptions = array();
            
    		foreach( \IPS\donate\Gateway::roots( NULL, NULL, array( 'gw_active=1' ) ) as $goal )
    		{
    			$gatewayOptions[ $goal->id ] = $goal->name;
    		} 
            
            /* We need at least 1 gateway to continue */
            if( !\count( $gatewayOptions ) )
            {                  
                \IPS\Output::i()->error( 'gateways_required', '', 403, 'gateways_required_admin' );
            }                        
            
            /* Add goal select */
            $form->add( new \IPS\Helpers\Form\Select( 'gateway', NULL, TRUE, array( 'options' => $gatewayOptions ) ) );

            /* Payment custom fields */
            $customFields = \IPS\donate\Field::roots();

            if( \count( $customFields ) )
            {
                /* Get any existing field values */
                try
                {
                    $fieldValues = \IPS\Db::i()->select( '*', 'donate_custom_values', array( 'invoice_id=?', $invoice->id ) )->first();
                }
                catch ( \UnderflowException $e ){}

                /* Show each custom field */
                foreach( $customFields as $id => $field )
                {
                    $value = ( isset( $fieldValues['field_'.$field->id ] ) ) ? $fieldValues['field_'.$field->id ] : NULL;
                    $form->add( $field->buildHelper( $value ) );
                }
            }

			if( $values = $form->values() )
			{
			    /* Format amount value */
			    $values['amount'] = new \IPS\donate\Currency\Money( $values['amount'], '' );
                $values['amount'] = (string) $values['amount'];

			    /* Check required fields */
                if( !$values['amount'] )
                {
                    $form->error = \IPS\Member::loggedIn()->language()->addToStack( 'amount_required' );
                } 
                if( !$values['currency'] )
                {
                    $form->error = \IPS\Member::loggedIn()->language()->addToStack( 'currency_required' );
                } 
                if( !$values['gateway'] )
                {
                    $form->error = \IPS\Member::loggedIn()->language()->addToStack( 'gateway_required' );
                }

                /* Check minimum value against currency */
                try
                {
                    $currency = \IPS\donate\Currency::load( $values['currency'], 'c_tag' );

                    /* Require minimum amount */
                    if( ( $values['amount'] * $currency->rate ) < \IPS\Settings::i()->dt_min_donation )
                    {
                        $form->error = \IPS\Member::loggedIn()->language()->addToStack( 'minimum_amount_required', FALSE, array( 'sprintf' => array( new \IPS\donate\Currency\Money( \IPS\Settings::i()->dt_min_donation, '-1' ) ) ) );
                    }
                }
                /* Have this as backup */
                catch ( \OutOfRangeException $e )
                {
                    /* Require minimum amount */
                    if( $values['amount'] < \IPS\Settings::i()->dt_min_donation )
                    {
                        $form->error = \IPS\Member::loggedIn()->language()->addToStack( 'minimum_amount_required', FALSE, array( 'sprintf' => array( new \IPS\donate\Currency\Money( \IPS\Settings::i()->dt_min_donation, '' ) ) ) );
                    }
                }
                           
                /* Any form errors? */
                if( isset( $form->error ) AND $form->error )
                {
                    return $form;   
                }

                /* Save custom fields */
                $customValues = array();

                foreach( \IPS\donate\Field::roots() as $field )
                {
                    $helper  = $field->buildHelper();
                    if( $helper instanceof \IPS\Helpers\Form\Upload )
                    {
                        $customValues[ "field_{$field->id}" ] = (string) $values[ "donate_donatefield_{$field->id}" ];
                    }
                    else
                    {
                        $customValues[ "field_{$field->id}" ] = $helper::stringValue( $values[ "donate_donatefield_{$field->id}" ] );
                    }

                    if( $helper instanceof \IPS\Helpers\Form\Editor )
                    {
                        $field->claimAttachments( $this->tid );
                    }
                }

                if ( !empty( $customValues ) )
                {
                    \IPS\Db::i()->insert( 'donate_custom_values', array_merge( array( 'invoice_id' => $invoice->id, 'updated' => time() ), $customValues ), TRUE );
                }

				return $values;
			}            
			
            return \IPS\Output::i()->output = $form;
		};
		
        /* Now lets get more info */
		$steps['payment_screen'] = function ( $data ) use ( $invoice )
		{
		    /* Get payment gateway */
            $gateway = \IPS\donate\Gateway::load( $data['gateway'] );
            
            /* Setup payment form */
    		$form = new \IPS\Helpers\Form( 'donation', ( $gateway->gatewayURL() !== NULL ) ? 'pay_now' : '', $gateway->gatewayURL() );
    		$form->class = 'ipsForm_fullWidth ipsPad';    
            $form->ajaxOutput = FALSE;

            /* Check minimum value against currency */
            try
            {
                $currency = \IPS\donate\Currency::load( $data['currency'], 'c_tag' );

                /* Update invoice with currency */
                $invoice->currency = $currency->id;

                /* Require minimum amount */
                if( ( $data['amount'] * $currency->rate ) < \IPS\Settings::i()->dt_min_donation )
                {
                    $form->error = \IPS\Member::loggedIn()->language()->addToStack( 'minimum_amount_required', FALSE, array( 'sprintf' => array( new \IPS\donate\Currency\Money( \IPS\Settings::i()->dt_min_donation, '-1' ) ) ) );
                }
            }
                /* Have this as backup */
            catch ( \OutOfRangeException $e )
            {
                /* Require minimum amount */
                if( $data['amount'] < \IPS\Settings::i()->dt_min_donation )
                {
                    $form->error = \IPS\Member::loggedIn()->language()->addToStack( 'minimum_amount_required', FALSE, array( 'sprintf' => array( new \IPS\donate\Currency\Money( \IPS\Settings::i()->dt_min_donation, '' ) ) ) );
                }
            }
        
            /* Add donation amount */
            $form->addDummy( 'donation_amount', new \IPS\donate\Currency\Money( $data['amount'], $data['currency'] ) );
            
            /* Add goal details */
            if( isset( $data['goal'] ) AND $data['goal'] )
            {
                $goal = \IPS\donate\Goal::load( $data['goal'] );
                $form->addDummy( 'donation_goal', $goal->_title );   
            }

            /* Payment custom fields */
            $customFields = \IPS\donate\Field::roots();

            if( \count( $customFields ) )
            {
                /* Get any existing field values */
                try
                {
                    $fieldValues = \IPS\Db::i()->select( '*', 'donate_custom_values', array( 'invoice_id=?', $invoice->id ) )->first();
                }
                catch ( \UnderflowException $e ){}

                /* Show each custom field */
                foreach( $customFields as $id => $field )
                {
                    $value = ( isset( $fieldValues['field_'.$field->id ] ) ) ? $fieldValues['field_'.$field->id ] : '--';
                    $form->addDummy( $field->_title, $value );
                }
            }
        
            /* Add gateway summary */
            if( $gateway->summary )
            {
                $form->addDummy( $gateway->name, $gateway->summary );
            } 
            
            /* Add Recipient Options */
            if( $gateway->emails )
            {
                $recipientEmails = json_decode( $gateway->emails );
                $emailOptions = array();
                
                foreach( $recipientEmails as $recipient )
                {
                    $emailOptions[ $recipient->value ] = $recipient->key;
                }
                
                $form->add( new \IPS\Helpers\Form\Select( 'business', FALSE, FALSE, array( 'options' => $emailOptions ), NULL, NULL, NULL, 'business' ) );             
            }

            /* Any form errors? */
            if( isset( $form->error ) AND $form->error )
            {
                \IPS\Output::i()->error( $form->error, '', 403, '' );
            }

            /* Update invoice with latest data */
            $invoice->amount = $data['amount'];
            $invoice->goal = $data['goal'];
            $invoice->gateway = $gateway->id;
            $invoice->save();

            /* Print payment screen */            
            return \IPS\Output::i()->output = $gateway->paymentScreen( $form, array(), $data, $invoice, $goal );
		};
        
		/* Display */
		\IPS\Output::i()->sidebar['enabled'] = FALSE;
		\IPS\Output::i()->bodyClasses[] = 'ipsLayout_minimal';        
		\IPS\Output::i()->title	= \IPS\Member::loggedIn()->language()->addToStack('make_donation');
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate('payment')->paymentWrapper( (string) new \IPS\Helpers\Wizard( $steps, \IPS\Http\Url::internal( "app=donate&module=donate&controller=payment", 'front', 'donate_donate' ), TRUE ) );    
	}
}