<?php
/**
 * @package		Donations
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */

namespace IPS\donate\setup\upg_100106;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * 3.4.4 Upgrade Code
 */
class _Upgrade
{
    /**
     * Update approved notification
     *
     * @return	array	If returns TRUE, upgrader will proceed to next step. If it returns any other value, it will set this as the value of the 'extra' GET parameter and rerun this step (useful for loops)
     */
    public function step1()
    {
        $perCycle	= 20;
        $did		= 0;
        $limit		= \intval( \IPS\Request::i()->extra );

        /* Try to prevent timeouts to the extent possible */
        $cutOff	= \IPS\core\Setup\Upgrade::determineCutoff();

        foreach( \IPS\Db::i()->select( '*', 'donate_users', null, 'id ASC', array( $limit, $perCycle ) ) as $donation )
        {
            if( $cutOff !== null AND time() >= $cutOff )
            {
                return ( $limit + $did );
            }

            $did++;

            /* Update approved notification */
            $donation = \IPS\donate\Donation::constructFromData( $donation );
            $donation->approved_notified = time();
            $donation->save();
        }

        if( $did )
        {
            return ( $limit + $did );
        }
        else
        {
            unset( $_SESSION['_step1Count'] );

            return TRUE;
        }
    }

    /**
     * Custom title for this step
     *
     * @return string
     */
    public function step1CustomTitle()
    {
        $limit = isset( \IPS\Request::i()->extra ) ? \IPS\Request::i()->extra : 0;

        if( !isset( $_SESSION['_step1Count'] ) )
        {
            $_SESSION['_step1Count'] = \IPS\Db::i()->select( 'COUNT(*)', 'donate_users' )->first();
        }

        return "Rebuilding donations (Completed so far: " . ( ( $limit > $_SESSION['_step1Count'] ) ? $_SESSION['_step1Count'] : $limit ) . ' out of ' . $_SESSION['_step1Count'] . ')';
    }
}