<?php
/**
 * @package		Donations
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */

namespace IPS\donate\setup\upg_100202;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * 3.5.2 Upgrade Code
 */
class _Upgrade
{
    /**
     * Convert donate permissions
     */
    public function step1()
    {
        $perCycle	= 20;
        $did		= 0;
        $limit		= \intval( \IPS\Request::i()->extra );

        /* Setup list of our permissions */
        $ourPermissions = array( 'g_dt_view', 'g_dt_donate', 'g_dt_view_goals', 'g_dt_view_donations', 'g_dt_view_topdonors', 'g_dt_send_donations', 'g_dt_moderate_donations' );

        /* Try to prevent timeouts to the extent possible */
        $cutOff	= \IPS\core\Setup\Upgrade::determineCutoff();

        foreach( \IPS\Db::i()->select( '*', 'core_groups', null, 'g_id ASC', array( $limit, $perCycle ) ) as $row )
        {
            if( $cutOff !== null AND time() >= $cutOff )
            {
                return ( $limit + $did );
            }

            $did++;

            /* Go through and setup permissions */
            $savePermissions = array();
            foreach( $ourPermissions as $perm )
            {
                $savePermissions[ $perm ] = ( isset( $row[ $perm ] ) ) ? (int) $row[ $perm ] : 0;
            }

            /* Now insert group permissions */
            \IPS\Db::i()->insert( 'donate_groups', array_merge( array( 'group_id' => $row['g_id'] ), $savePermissions ), TRUE );
        }

        if( $did )
        {
            return ( $limit + $did );
        }
        else
        {
            unset( $_SESSION['_step1Count'] );
            return TRUE;
        }
    }

    /**
     * Custom title for this step
     *
     * @return string
     */
    public function step1CustomTitle()
    {
        $limit = isset( \IPS\Request::i()->extra ) ? \IPS\Request::i()->extra : 0;

        if( !isset( $_SESSION['_step1Count'] ) )
        {
            $_SESSION['_step1Count'] = \IPS\Db::i()->select( 'COUNT(*)', 'core_groups' )->first();
        }

        return "Converting donate group permissions (Rebuild so far: " . ( ( $limit > $_SESSION['_step1Count'] ) ? $_SESSION['_step1Count'] : $limit ) . ' out of ' . $_SESSION['_step1Count'] . ')';
    }

    /**
     * Adjust/Remove legacy fields
     *
     * @return	array	If returns TRUE, upgrader will proceed to next step. If it returns any other value, it will set this as the value of the 'extra' GET parameter and rerun this step (useful for loops)
     */
    public function step2()
    {
        /* Trigger background queue to remove legacy fields */
        try
        {
            \IPS\Task::queue( 'donate', 'membersLegacyField', array(), 1 );
        }
        catch ( \OutOfRangeException $ex ) { }

        return TRUE;
    }

    /**
     * Remove old group permissions
     *
     * @return	array	If returns TRUE, upgrader will proceed to next step. If it returns any other value, it will set this as the value of the 'extra' GET parameter and rerun this step (useful for loops)
     */
    public function step3()
    {
        /* Setup list of our permissions */
        $ourPermissions = array( 'g_dt_view', 'g_dt_donate', 'g_dt_view_goals', 'g_dt_view_donations', 'g_dt_view_topdonors', 'g_dt_send_donations', 'g_dt_moderate_donations' );

        /* Remove old permission fields */
        foreach( $ourPermissions as $perm )
        {
            if( \IPS\Db::i()->checkForColumn( 'core_groups', $perm ) )
            {
                \IPS\Db::i()->dropColumn( 'core_groups', array( $perm ) );
            }
        }

        return true;
    }
}