<?php
/**
 * @package		Donations
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */

namespace IPS\donate;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Field Node
 */
class _Field extends \IPS\CustomField
{
	/**
	 * @brief	[ActiveRecord] Multiton Store
	 */
	protected static $multitons;
	
	/**
	 * @brief	[ActiveRecord] Database Table
	 */
	public static $databaseTable = 'donate_custom_fields';
	
	/**
	 * @brief	[ActiveRecord] Database Prefix
	 */
	public static $databasePrefix = 'cf_';
		
	/**
	 * @brief	[Node] Order Database Column
	 */
	public static $databaseColumnOrder = 'position';

	/**
	 * @brief	[CustomField] Column Map
	 */
	public static $databaseColumnMap = array(
		'not_null'	=> 'not_null'
	);
	
	/**
	 * @brief	[Node] Node Title
	 */
	public static $nodeTitle = 'donate_custom_fields';
	
	/**
	 * @brief	[CustomField] Title/Description lang prefix
	 */
	protected static $langKey = 'donate_donatefield';

	/**
	 * @brief	[Node] Title prefix.  If specified, will look for a language key with "{$key}_title" as the key
	 */
	public static $titleLangPrefix = 'donate_donatefield_';
	
	/**
	 * @brief	[CustomField] Content Table
	 */
	public static $contentDatabaseTable = 'donate_custom_values';
	
	/**
	 * @brief	[Node] ACP Restrictions
	 */
	protected static $restrictions = array(
		'app'		=> 'donate',
		'module'	=> 'setup',
		'prefix'	=> 'donatefields_',
	);

	/**
	 * @brief	[CustomField] Editor Options
	 */
	public static $editorOptions = array( 'app' => 'donate', 'key' => 'Fields' );
	
	/**
	 * @brief	[CustomField] FileStorage Extension for Upload fields
	 */
	public static $uploadStorageExtension = 'donate_DonateField';

    /**
     * Get field data
     *
     * @return	array
     */
    protected static function fieldData()
    {
        if ( !isset( \IPS\Data\Store::i()->donateFields ) )
        {
            $fields = array();

            foreach ( \IPS\Db::i()->select( '*', 'donate_custom_fields', NULL, 'cf_position' ) as $row )
            {
                $fields[ $row['cf_id'] ] = $row;
            }

            \IPS\Data\Store::i()->donateFields = $fields;
        }

        return \IPS\Data\Store::i()->donateFields;
    }

    /**
     * Get Fields
     *
     * @param	array		$values		Current values
     * @return	array
     */
    public static function fields( $values=array() )
    {
        if( !$values )
        {
            $values = array();
        }

        $return = array();

        foreach ( static::fieldData() as $tabId => $fields )
        {
            foreach ( $fields as $row )
            {
                if ( !array_key_exists( 'field_' . $row['field_id'], $values ) )
                {
                    $values['field_' . $row['field_id'] ] = NULL;
                }

                static::$editorOptions['autoSaveKey'] = md5( \get_called_class() . '-' . $row['field_id'] ) . ( isset( \IPS\Request::i()->id ) ? '-' . \IPS\Request::i()->id : '' );

                $return[ $tabId ][ $row['field_id'] ] = static::constructFromData( $row )->buildHelper( $values[ 'field_' . $row['field_id'] ] );
            }
        }

        return $return;
    }

    /**
     * [Node] Add/Edit Form
     *
     * @param	\IPS\Helpers\Form	$form	The form
     * @return	void
     */
    public function form( &$form )
    {
        parent::form( $form );

        /* Add fields */
        $form->add( new \IPS\Helpers\Form\YesNo( 'cf_display', ( $this->id ) ? $this->display : TRUE ) );
        $form->add( new \IPS\Helpers\Form\Text( 'cf_value', ( $this->id ) ? $this->value : '' ) );

        /* Make required field false by default */
        $form->elements['']['pf_not_null']->value = ( $this->id ) ? $form->elements['']['pf_not_null']->value : FALSE;

        /* Remove uneeded fields */
        unset( $form->elements[''][1] );
        unset( $form->elements['']['pf_search_type'] );
        unset( $form->elements['']['pf_format'] );
        unset( $form->elements['']['pf_search_type_on_off'] );
        unset( $form->elements['']['pf_allow_attachments'] );
    }

    /**
     * [Node] Format form values from add/edit form for save
     *
     * @param	array	$values	Values from the form
     * @return	array
     */
    public function formatFormValues( $values )
    {
        return parent::formatFormValues( $values );
    }

    /**
     * Build Form Helper
     *
     * @param	mixed	$value	                    The value
     * @param	callback	$customValidationCode	Custom validation code
     * @param   \IPS\cms\Records|NULL   $content     The record
     * @param	int				        $flags		Bit flags
     * @return \IPS\Helpers\Form\FormAbstract
     */
    public function buildHelper( $value=NULL, $customValidationCode=NULL, \IPS\Content $content = NULL, $flags=0 )
    {
        /* Add default field value */
        if( !$value AND $this->value )
        {
            $value = $this->value;
        }

        return parent::buildHelper( $value );
    }
}