<?php

namespace IPS\donate;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Invoice
 */
class _Invoice extends \IPS\Patterns\ActiveRecord
{
	/**
	 * @brief	[ActiveRecord] Multiton Store
	 */
	protected static $multitons;    
    
	/**
	 * @brief	[ActiveRecord] Database Table
	 */
	public static $databaseTable = 'donate_invoices';
	
	/**
	 * @brief	[ActiveRecord] Database Prefix
	 */
	public static $databasePrefix = 'invoice_';
    
	/**
	 * @brief	[ActiveRecord] ID Database Column
	 */
	public static $databaseColumnId = 'id';

    /**
     * @brief	[ActiveRecord] Database ID Fields
     */
    protected static $databaseIdFields = array('invoice_id');

    /**
     * @brief	Cached URL
     */
    protected $_url	= NULL;

    /**
     * Get URL
     *
     * @return	\IPS\Http\Url
     */
    public function url()
    {
        if( $this->_url === NULL )
        {
            $this->_url = \IPS\Http\Url::internal("app=donate&module=donate&controller=payment", 'front', 'donate_donate' );
        }

        return $this->_url;
    }

    /**
	 * [ActiveRecord] Save Record
	 *
	 * @return	void
	 */
	public function save()
	{
        /* Lets assume this is always the same. */
        if( !$this->member )
        {
            $this->member = \IPS\Member::loggedIn()->member_id;
        }
        if( !$this->ip )
        {
            $this->ip = \IPS\Request::i()->ipAddress();
        }
        if( !$this->date )
        {
            $this->date = time();
        }
        if( !$this->status )
        {
            /* Unpaid */
            $this->status = 0;
        }

        return parent::save();
	}

    /**
     * Get currency
     *
     */
    public function currency()
    {
        if ( $this->currency )
        {
            try
            {
                return \IPS\donate\Currency::load( $this->currency );
            }
            catch ( \OutOfRangeException $e ) { }
        }

        return NULL;
    }
}