<?php
/**
 * @package		Donations
 * @author		<a href='https://www.devfuse.com'>DevFuse</a>
 * @copyright	(c) 2021 DevFuse
 */

namespace IPS\donate\widgets;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * donateStats Widget
 */
class _donateStats extends \IPS\Widget\PermissionCache
{
	/**
	 * @brief	Widget Key
	 */
	public $key = 'donateStats';
	
	/**
	 * @brief	App
	 */
	public $app = 'donate';
		
	/**
	 * @brief	Plugin
	 */
	public $plugin = '';
	
	/**
	 * Initialise this widget
	 *
	 * @return void
	 */ 
	public function init()
	{
		parent::init();
	}

	/**
	 * Render a widget
	 *
	 * @return	string
	 */
	public function render()
	{
	    /* No permission to view */
        if( !\IPS\Member::loggedIn()->group['g_dt_view_donations'] )
        {
            return '';
        }

	    $stats = array();

        /* Get donation totals */
        if( \count( \IPS\donate\Goal::roots() ) )
        {
            $donationWhere[] = array( 'donate_users.status=? AND donate_goals.g_show=?', 1, 1 );
        }
        else
        {
            $donationWhere[] = array( 'donate_users.status=?', 1 );
        }

        /* Automatic reset? */
        if( \IPS\Settings::i()->dt_donation_reset )
        {
            $donationWhere[] = array( 'date>?', (int) \IPS\Settings::i()->dt_donation_reset );    
        }      

        $stats['totalDonations'] = \IPS\Db::i()->select( 'SUM(amount * rate)', 'donate_users', $donationWhere )->join(
					'donate_goals',
					'donate_goals.g_id=donate_users.goal'
			)->first();

        $stats['totalDonors'] = \IPS\Db::i()->select( 'COUNT(DISTINCT(member_id))', 'donate_users', $donationWhere )->join(
            'donate_goals',
            'donate_goals.g_id=donate_users.goal'
        )->first();

        /* Get fee totals */
        if( \IPS\Settings::i()->dt_include_fees )
        {
            $stats['totalFees'] = \IPS\Db::i()->select( 'SUM(fees)', 'donate_users', $donationWhere )->join(
					'donate_goals',
					'donate_goals.g_id=donate_users.goal'
			)->first();  
            
            /* Take out fees for donation total */
            $stats['totalDonations'] = $stats['totalDonations'] - $stats['totalFees'];
        }
                
        /* Get goal totals */
        if( \IPS\Settings::i()->dt_goal_total )
        {
            $stats['totalGoals'] = \IPS\Settings::i()->dt_goal_total;    
        }
        else
        {
            $goalWhere[] = array( 'g_show=?', 1 );

            $stats['totalGoals'] = \IPS\Db::i()->select( 'SUM(g_amount)', 'donate_goals', $goalWhere )->first();     
        }

        /* Get latest donor */
        $latestDonor = NULL;

        if( \IPS\Member::loggedIn()->group['g_dt_view_donations'] )
        {
            foreach ( \IPS\donate\Donation::getItemsWithPermission( array( array( 'status=1' ) ), 'date DESC', 1 ) as $latestDonor )
            {
                break;
            }
        }


		return $this->output( $stats, $latestDonor );
	}
}